function preloader() {
	function getCssImages() {
		/**
		 * Given a CSS URL value, returns the actual value 
		 */
		function extractBackgroundImage(cssRule) {
			var url = cssRule.style.backgroundImage;
			url = url.replace("url(", "");
			url = url.replace(/"/g, "");
			url = url.replace(")", "");

			if(url.indexOf("http") != 0) {
				var base = cssRule.parentStyleSheet.href;
				if(base == null) return "";

				base = base.split("/").slice(0, -1).join("/");
				url = base + "/" + url;
			}

			return url;
		}

		/**
		 * When accessing cssRules for a file wieh CORS issue, Firefox throws a error,
		 * chrome returns null.
		 *
		 * Use this function to get the same behaviour
		 */
		function getCssRules(styleSheet) {
			var rules = null;
			try { rules = styleSheet.cssRules } catch(error) {};
			return rules;
		}

		function processStyleSheet(styleSheet) {
			// Styles cross-site can not be processed
			var result = [];
			var cssRules = getCssRules(styleSheet);
			if (cssRules == null) return result;

			var cssRulesLenght = cssRules.length;
			for(var ruleCount = 0; ruleCount < cssRulesLenght; ruleCount++) {
				var cssRule = cssRules[ruleCount];

				if(cssRule instanceof CSSMediaRule) continue;
				else if(cssRule instanceof CSSImportRule) {
					result.push.apply(result, processStyleSheet(cssRule.styleSheet));
				}
				else {
					if(cssRule.style.backgroundImage != "") {
						var image = extractBackgroundImage(cssRule);
						if(image != "") result.push(image);
					}
				}
			}

			return result
		}

		var cssImages = [];
		var styleSheetsLenght = document.styleSheets.length;
		for(var count = 0; count < styleSheetsLenght; count++) {
			var styleSheet = document.styleSheets[count];
			cssImages.push.apply(cssImages, processStyleSheet(styleSheet));
		}

		return cssImages;
	}

	var images = getCssImages();
	var promisesObjects = [];

	for (var count = 0; count < images.length; count++) {
		var imageDeferred = $.Deferred();
		var resolve = function() { this.resolve(); }.bind(imageDeferred);

		var imageObject = new Image();
		imageObject.src = images[count];
		imageObject.addEventListener("load", resolve);
		imageObject.addEventListener("error", resolve);

		promisesObjects.push(imageDeferred);
	}

	var result = $.when.apply($, promisesObjects)
	return result.promise();
} 


/** 
 * El siguiente código Inicializa el preloading y crea los datos necesarios en la pagina 
 * Se ejecuta sincronicamente.
 **/
(function init_page() {
	var strAnimationScript = '<script src="../assets-common/javascript/preloading.js"></script>';
	var strAnimationStyles = '<link rel="stylesheet" href="../assets-common/styles/preloading.css" type="text/css">';
	var strPreloadingHtml = '<div id="preloading"><div id="preloading_asteroide1"></div><div id="preloading_asteroide2"></div><div id="preloading_planeta1"></div><div id="preloading_planeta2"></div><div id="preloading_planeta3"></div><div id="preloading_monstruo"></div><div id="preloading_cargando"></div><div id="preloading_nave"></div></div>';

	// Agregamos el CSS del Preloading. Como esto se ejecuta concurrentemente en la página. Lo escribimos
	// directamente en el documento.
	document.write(strAnimationStyles);
	document.write(strPreloadingHtml);
	document.write(strAnimationScript);

	$("#container").addClass("preloading");

	// Comenzamos la carga de todas las imágenes, y cuando termina ocultamos el preloading.
	preloader().done(function() {
		$("#container").removeClass("preloading");
		$("#preloading").remove()
	});	
})();